﻿/*
@file    tft.c
@brief   Testing UniCode Font with BT815
@version 1.0
@date    2019-04-25
@author  Rudolph Riedel

@section History

1.0
Derived from older tft.c Test-Code
 
 */

#include "EVE.h"
#include "EVE_commands.h"

/* some pre-definded colors */
#define RED		0xff0000UL
#define ORANGE	0xffa500UL
#define GREEN	0x00ff00UL
#define BLUE	0x0000ffUL
#define BLUE_1	0x5dade2L
#define YELLOW	0xffff00UL
#define PINK	0xff00ffUL
#define PURPLE	0x800080UL
#define WHITE	0xffffffUL
#define BLACK	0x000000UL

/* memory-map defines */
#define MEM_LOGO 147456 /* start-address of "chipsie_40x56_40x56_COMPRESSED_RGBA_ASTC_8x8_KHR.raw" in FLASH */
#define MEM_PIC1 148032 /* start-address 0f "Circles_64x64_COMPRESSED_RGBA_ASTC_8x8_KHR.raw " in FLASH */
#define MEM_FONT_FLASH 142592 /* start-address of "GlacialIndifference-Regular_20.xfont" in FLASH */

#define MEM_FONT_STRUCT 0xfd000 /* start-address of GlacialIndifference-Regular_20.xfont in G-RAM */
#define FONT_STRUCT_SIZE 4864 /* size of "GlacialIndifference-Regular_20.xfont" */

#define MEM_DL_STATIC (EVE_RAM_G_SIZE - 4096) /* start-address of the static part of the display-list, upper 4k of gfx-mem */

uint32_t num_dl_static; /* amount of bytes in the static part of our display-list */
uint8_t tft_active = 0;
uint16_t num_profile_a, num_profile_b;

#define LAYOUT_Y1 66


void touch_calibrate(void)
{

/* send pre-recorded touch calibration values, depending on the display the code is compiled for */

#if defined (EVE_RiTFT43)
	EVE_memWrite32(REG_TOUCH_TRANSFORM_A, 0x000063a2);
	EVE_memWrite32(REG_TOUCH_TRANSFORM_B, 0xffffff94);
	EVE_memWrite32(REG_TOUCH_TRANSFORM_C, 0xfff4f2d9);
	EVE_memWrite32(REG_TOUCH_TRANSFORM_D, 0xffffffe0);
	EVE_memWrite32(REG_TOUCH_TRANSFORM_E, 0x000058b3);
	EVE_memWrite32(REG_TOUCH_TRANSFORM_F, 0x0004e676);
#endif

/* activate this if you are using a module for the first time or if you need to re-calibrate it */
/* write down the numbers on the screen and either place them in one of the pre-defined blocks above or make a new block */
#if 0
	/* calibrate touch and displays values to screen */
	EVE_cmd_dl(CMD_DLSTART);
	EVE_cmd_dl(DL_CLEAR_RGB | BLACK);
	EVE_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG);
	EVE_cmd_text((EVE_HSIZE/2), 50, 26, EVE_OPT_CENTER, "Please tap on the dot.");
	EVE_cmd_calibrate();
	EVE_cmd_dl(DL_DISPLAY);
	EVE_cmd_dl(CMD_SWAP);
	EVE_cmd_execute();

	uint32_t touch_a, touch_b, touch_c, touch_d, touch_e, touch_f;

	touch_a = EVE_memRead32(REG_TOUCH_TRANSFORM_A);
	touch_b = EVE_memRead32(REG_TOUCH_TRANSFORM_B);
	touch_c = EVE_memRead32(REG_TOUCH_TRANSFORM_C);
	touch_d = EVE_memRead32(REG_TOUCH_TRANSFORM_D);
	touch_e = EVE_memRead32(REG_TOUCH_TRANSFORM_E);
	touch_f = EVE_memRead32(REG_TOUCH_TRANSFORM_F);

	EVE_cmd_dl(CMD_DLSTART);
	EVE_cmd_dl(DL_CLEAR_RGB | BLACK);
	EVE_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG);
	EVE_cmd_dl(TAG(0));

	EVE_cmd_text(5, 15, 26, 0, "TOUCH_TRANSFORM_A:");
	EVE_cmd_text(5, 30, 26, 0, "TOUCH_TRANSFORM_B:");
	EVE_cmd_text(5, 45, 26, 0, "TOUCH_TRANSFORM_C:");
	EVE_cmd_text(5, 60, 26, 0, "TOUCH_TRANSFORM_D:");
	EVE_cmd_text(5, 75, 26, 0, "TOUCH_TRANSFORM_E:");
	EVE_cmd_text(5, 90, 26, 0, "TOUCH_TRANSFORM_F:");

#if defined (FT81X_ENABLE)
	EVE_cmd_setbase(16L); /* FT81x only */
	EVE_cmd_number(310, 15, 26, EVE_OPT_RIGHTX|8, touch_a);
	EVE_cmd_number(310, 30, 26, EVE_OPT_RIGHTX|8, touch_b);
	EVE_cmd_number(310, 45, 26, EVE_OPT_RIGHTX|8, touch_c);
	EVE_cmd_number(310, 60, 26, EVE_OPT_RIGHTX|8, touch_d);
	EVE_cmd_number(310, 75, 26, EVE_OPT_RIGHTX|8, touch_e);
	EVE_cmd_number(310, 90, 26, EVE_OPT_RIGHTX|8, touch_f);
#else
	EVE_cmd_number(310, 15, 26, EVE_OPT_RIGHTX, touch_a);
	EVE_cmd_number(310, 30, 26, EVE_OPT_RIGHTX, touch_b);
	EVE_cmd_number(310, 45, 26, EVE_OPT_RIGHTX, touch_c);
	EVE_cmd_number(310, 60, 26, EVE_OPT_RIGHTX, touch_d);
	EVE_cmd_number(310, 75, 26, EVE_OPT_RIGHTX, touch_e);
	EVE_cmd_number(310, 90, 26, EVE_OPT_RIGHTX, touch_f);
#endif

	EVE_cmd_dl(DL_DISPLAY);	/* instruct the graphics processor to show the list */
	EVE_cmd_dl(CMD_SWAP); /* make this list active */
	EVE_cmd_execute();

	while(1);
#endif
}


void initStaticBackground()
{
	EVE_cmd_dl(CMD_DLSTART); /* Start the display list */
	
	EVE_cmd_dl(TAG(0)); /* do not use the following objects for touch-detection */

	EVE_cmd_bgcolor(0x00c0c0c0); /* light grey */

	EVE_cmd_dl(VERTEX_FORMAT(0)); /* reduce precision for VERTEX2F to 1 pixel instead of 1/16 pixel default */

	/* draw a rectangle on top */
	EVE_cmd_dl(DL_BEGIN | EVE_RECTS);
	EVE_cmd_dl(LINE_WIDTH(1*16)); /* size is in 1/16 pixel */

	EVE_cmd_dl(DL_COLOR_RGB | BLUE_1);
	EVE_cmd_dl(VERTEX2F(0,0));
	EVE_cmd_dl(VERTEX2F(EVE_HSIZE,LAYOUT_Y1-2));
	EVE_cmd_dl(DL_END);

	/* display the logo */
	EVE_cmd_dl(DL_COLOR_RGB | WHITE);
	EVE_cmd_dl(DL_BEGIN | EVE_BITMAPS);
	EVE_cmd_setbitmap(0x800000 | (MEM_LOGO/32), EVE_COMPRESSED_RGBA_ASTC_8x8_KHR, 40, 56);
	EVE_cmd_dl(VERTEX2F(EVE_HSIZE - 50, 5));
	EVE_cmd_dl(DL_END);

	/* draw a black line to separate things */
	EVE_cmd_dl(DL_COLOR_RGB | BLACK);
	EVE_cmd_dl(DL_BEGIN | EVE_LINES);
	EVE_cmd_dl(VERTEX2F(0,LAYOUT_Y1-2));
	EVE_cmd_dl(VERTEX2F(EVE_HSIZE,LAYOUT_Y1-2));
	EVE_cmd_dl(DL_END);

	/* add the static text to the list */
	EVE_cmd_text(10, EVE_VSIZE - 65, 26, 0, "DL-size:");
	EVE_cmd_text(10, EVE_VSIZE - 50, 26, 0, "Bytes:");
	EVE_cmd_text(10, EVE_VSIZE - 35, 26, 0, "Time1:");
	EVE_cmd_text(10, EVE_VSIZE - 20, 26, 0, "Time2:");

	EVE_cmd_text(125, EVE_VSIZE - 35, 26, 0, "us");
	EVE_cmd_text(125, EVE_VSIZE - 20, 26, 0, "us");

	EVE_cmd_setfont2(14,MEM_FONT_STRUCT,32); /* assign bitmap handle 14 to a custom font */

	EVE_cmd_execute();

	num_dl_static = EVE_memRead16(REG_CMD_DL);

	EVE_cmd_memcpy(MEM_DL_STATIC, EVE_RAM_DL, num_dl_static);
	EVE_cmd_execute();
}


void TFT_init(void)
{
	if(EVE_init() != 0)
	{
		tft_active = 1;
		
		EVE_init_flash(); /* sets the FLASH to fast-mode */

		EVE_memWrite8(REG_PWM_DUTY, 0x30);	/* setup backlight, range is from 0 = off to 0x80 = max */
		touch_calibrate();
		
		EVE_cmd_flashread(MEM_FONT_STRUCT, MEM_FONT_FLASH, FONT_STRUCT_SIZE); /* copy GlacialIndifference-Regular_20.xfont from FLASH to G-RAM */

		initStaticBackground();
	}
}


uint16_t toggle_state = EVE_OPT_FLAT;

/* check for touch events and setup vars for TFT_display() */
void TFT_touch(void)
{
	uint32_t calc;
	uint8_t tag;
	static uint8_t toggle_lock = 0;

	if(EVE_busy()) /* is the FT8xx still processing the last display list? */
	{
		return;
	}

	calc = EVE_get_touch_tag(0);
	tag = calc;

	switch(tag)
	{
		case 0:
			toggle_lock = 0;
			break;

		case 1: /* use button on top as on/off radio-switch */
			if(toggle_lock == 0)
			{
				toggle_lock = 42;
				if(toggle_state == 0)
				{
					toggle_state = EVE_OPT_FLAT;
				}
				else
				{
					toggle_state = 0;
				}
			}
			break;
	}
}


/*
	dynamic portion of display-handling, meant to be called every 20ms or more
*/
void TFT_display(void)
{
    uint32_t calc;
	uint16_t old_offset, new_offset;

	uint16_t display_list_size = 0;

	if(tft_active != 0)
	{
		old_offset =  EVE_report_cmdoffset(); /* used to calculate the amount of cmd-fifo bytes necessary */
		display_list_size = EVE_memRead16(REG_CMD_DL);

		EVE_start_cmd_burst(); /* If DMA is enabled, this will initiate sending a block of data with DMA */

		EVE_cmd_dl(CMD_DLSTART); /* start the display list */
		EVE_cmd_dl(DL_CLEAR_RGB | WHITE); /* set the default clear color to white */
		EVE_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG); /* clear the screen - this and the previous prevent artifacts between lists, Attributes are the color, stencil and tag buffers */
		EVE_cmd_dl(TAG(0));

		EVE_cmd_append(MEM_DL_STATIC, num_dl_static); /* insert static part of display-list from copy in gfx-mem */

		EVE_cmd_dl(DL_COLOR_RGB | BLACK);

		EVE_cmd_text(20,100,14,0, "Hänsel macht bei 0° die Tür zu.");

		/* print profiling values */
				
		EVE_cmd_number(120, EVE_VSIZE - 65, 26, EVE_OPT_RIGHTX, display_list_size); /* number of bytes written to the display-list by the command co-pro */
		EVE_cmd_number(120, EVE_VSIZE - 35, 26, EVE_OPT_RIGHTX|5, num_profile_a); /* duration in µs of TFT_loop() for the touch-event part */
		EVE_cmd_number(120, EVE_VSIZE - 20, 26, EVE_OPT_RIGHTX|5, num_profile_b); /* duration in µs of TFT_loop() for the display-list part */

		new_offset =  EVE_report_cmdoffset();
		if(old_offset > new_offset)
		{
			new_offset+=4096;
		}
		calc = new_offset-old_offset;
		calc += 24; /* adjust for the commands that follow before the end */
		EVE_cmd_number(120, EVE_VSIZE - 50, 26, EVE_OPT_RIGHTX, calc); /* number of bytes written to the cmd-fifo over the spi without adressing overhead */

		EVE_cmd_dl(DL_DISPLAY);	/* instruct the graphics processor to show the list */
		EVE_cmd_dl(CMD_SWAP); /* make this list active */

		EVE_end_cmd_burst(); /* if DMA is enabled, this will start the transfer of the prepared buffer */
		EVE_cmd_start(); /* order the command co-processor to start processing its FIFO queue but do not wait for completion */
	}
}
